﻿using System;
using System.Collections.Generic;
using System.Collections.Immutable;
using System.Diagnostics;
using System.IO;
using System.Linq;
using System.Threading.Tasks;

namespace PickMe2021
{
    class Program
    {
        static IDictionary<string, string> defaultArgs = new Dictionary<string, string>() {
            {"dir", @"./" },
            {"times", "5" },
            {"extensions", "" }
        };

        static Dictionary<string, string> ParseArgs(IList<string> args)
        {
            var ret = defaultArgs.ToDictionary(x => x.Key, x => x.Value);
            for (var i = 0; i < args.Count(); ++i)
            {
                switch (args[i])
                {
                    case "-d":
                    case "-D":
                        ret["dir"] = args[i++ + 1];
                        break;
                    case "-t":
                    case "-T":
                        ret["times"] = args[i++ + 1];
                        break;
                    case "-e":
                    case "-E":
                        ret["extensions"] = args[i++ + 1];
                        break;
                    default:
                        var errorTxt = $"Command {args[i]} not understood";
                        ret["error"] = ret.ContainsKey("error") ? $"{ret["error"]}\n{errorTxt}" : errorTxt;
                        while (!args[i].StartsWith("-") && args.Count() < i) { ++i; }
                        break;
                }
            }
            return ret;
        }

        static IEnumerable<FileInfo> FilterFiles(IEnumerable<FileInfo> files, string allowedExtensions)
        {
            var extensions = allowedExtensions.Split(",").Where(x => x != "").Select(x => x.StartsWith(".") ? x.Trim() : $".{x.Trim()}");
            if (extensions.Count() != 0)
            {
                return files.Where(x => extensions.Any(y => x.Name.EndsWith(y)));
            }
            else
            {
                return files;
            }

        }

        static IList<int> GetRandomIndexes<T>(int times, IEnumerable<T> collection)
        {
            var amountOfFiles = Math.Min(times, collection.Count());

            var rand = new Random((int)DateTime.UtcNow.Ticks);
            var indexes = new List<int>();
            for (var i = 0; i < amountOfFiles; ++i)
            {
                int randIndex = 0;
                do
                {
                    randIndex = rand.Next(0, collection.Count());
                } while (indexes.Contains(randIndex));
                indexes.Add(randIndex);
            }
            return indexes;
        }

        static void Main(string[] args)
        {
            var options = ParseArgs(args);

            if (options.ContainsKey("error"))
            {
                Console.WriteLine(options["error"]);
                return;
            }

            DirectoryInfo rootdir = new DirectoryInfo(options["dir"]);

            if (!rootdir.Exists)
            {
                Console.WriteLine("Selected directory does not exist");
                return;
            }


            IEnumerable<FileInfo> files = null;
            try
            {
                files = DirectoryTraverse.TraverseDirectory(rootdir);
            }
            catch(UnauthorizedAccessException e)
            {
                Console.WriteLine($"{e.Message}");
                return;
            }

            var filteredFiles = FilterFiles(files, options["extensions"]).ToList();

            if (!filteredFiles.Any())
            {
                Console.WriteLine("No files found with the extension specified");
                return;
            }


            var randomIndexes = GetRandomIndexes(int.Parse(options["times"]), filteredFiles);

            for (var i = 0; i < randomIndexes.Count(); ++i)
            {
                Console.WriteLine($"{i} : {filteredFiles[randomIndexes[i]].Name}");
            }

            var input = 0;
            if (randomIndexes.Count() < 10)
            {
                var key = Console.ReadKey().KeyChar.ToString();
                input = int.Parse(key);
            }
            else
            {
                var inputString = Console.ReadLine();
                input = int.Parse(inputString);
            }

            if (input >= randomIndexes.Count())
            {
                Console.WriteLine("Selected value is out of range");
                return;
            }

            DirectoryTraverse.OpenExplorerToFile(filteredFiles[randomIndexes[input]]);
        }
    }
}
